/* 
 * Copyright (c) 2008 - Peter Jamieson
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * */
/*  Bit Vector Definitions */

#ifndef BITVECTOR_H
#define BITVECTOR_H

#include "lib_includes.h"

/*  Bit vectors are stored in arrays of unsigned integers.  The bits are
    numbered beginning with 0.  */
typedef int digital_bit;   

typedef struct bit_vector_struct {
	int num_bits;
	unsigned *bits;
} bit_vector;

typedef void (*bitprint_f)(FILE *fd, int bit);

extern bit_vector *new_bit_vector(int num_bits);
extern void set_bit(bit_vector *b, int ndx, digital_bit value);
extern digital_bit get_bit(bit_vector *b, int ndx);
extern void set_all_bits(bit_vector *b, digital_bit value);
extern void subtract_bits(bit_vector *b, bit_vector *c);
extern void and_bits(bit_vector *b, bit_vector *c);
extern void or_bits(bit_vector *b, bit_vector *c);
extern void not_bits (bit_vector *b);
extern void xor_bits (bit_vector *b, bit_vector *c);
extern void swap_vectors (bit_vector *b, bit_vector *c);
extern void copy_bits(bit_vector *b, bit_vector *c);
extern digital_bit bits_are_equal(bit_vector *b, bit_vector *c);
extern digital_bit bits_are_false(bit_vector *b);
extern digital_bit bits_are_true (bit_vector *b);
extern void fprint_bits(FILE *fd, bit_vector *b, bitprint_f bitprint);
extern void fprint_bits_out(FILE *fd, bit_vector *b, bitprint_f bitprint);
extern void fprint_bit(FILE *fd, int bit);
extern void free_bit_vector(bit_vector *b);
extern void set_bits_with_short_at(bit_vector *b, int ndx, unsigned short value);
extern void set_bits_with_short_with_mask_at(bit_vector *b, int ndx, unsigned short value, unsigned short mask);
extern unsigned short get_short_from_bits_at(bit_vector *b, int ndx, int number);
extern void copy_n_bits_at_x_to_y(int nbits, bit_vector *from, int starting_at_bit_from, bit_vector *to, int starting_at_bit_to);
extern void reverse_bits(bit_vector *vector);
extern digital_bit bits_are_equal_n_bits (bit_vector *b, bit_vector *c, int n);
extern void set_n_bits_with_int(bit_vector *b, unsigned int value, int n);
extern int get_int_from_bit_vector(bit_vector *b);

#endif /* BITVECTOR_H */
