/* 
 * Copyright (c) 2008 - Peter Jamieson
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * */
#include "bitvector.h"

void golden_functional_model_for_2dconv(workload_t *w, char *output_file);

#define conv_device_state_t struct conv_device_state_t_t

#ifndef LZIV_D
#define LZIV_D

// 4096 symbol is reserved for termination
#define MATRIX_SIZE 400
#define CONVOLUTION_MATRIX_SIZE 5

typedef union FIXED8_8tag // FIXEDX_Ytag
{
	long int full; // FULLSIZEINT
	struct parx_Ytag
	{
		int fraction: 8;  // X
		int integer: 8; // FULLSIZEINT-X
	} part;
} FIXED8_8; // FIXEDX_Y

// From FIXEDX_YCONST(A,B) (FULLSIZEINT)((A<<Y)+((B+1/(2^(Y+1)))*2^Y))
#define FIXED8_8CONST(A,B) (long int)((A<<8)+((B+0.0078125)*256))
// From MULTX_Y(A,B) (FULLSIZEINT+1)(A.full*B.full+2^(Y-1))>>Y
#define MULT8_8(A,B) (long int)((A.full*B.full+ 128) >> 8)
#define OVERFLOW(A) (long int)(A.full & 65535)

#define FIXEDPOINT_CALC
//#define INT_CALC

conv_device_state_t
{
#ifdef INT_CALC
	int **matrix;
	int **out_matrix;
	int **convolution_matrix;
#endif
#ifdef FIXEDPOINT_CALC
	FIXED8_8 **matrix;
	FIXED8_8 **out_matrix;
	FIXED8_8 **convolution_matrix;
#endif
	short is_convolution_matrix;
	int matrix_index;
	int matrix_process_index;
};

#endif
