/* 
 * Copyright (c) 2008 - Peter Jamieson
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * */
/* keypress and release code with simple addition of time */
{
		int row = my_rand()%4;
		int column = my_rand()%4;
		mpz_t *time_to_add;

		assert(wave->num_workload_value == 1);

		value = (char**)malloc(sizeof(char*)*2);
		value[0] = (char*)malloc(sizeof(char)*2);
		value[1] = (char*)malloc(sizeof(char)*2);
		sprintf(value[0], "%d", row);
		sprintf(value[1], "%d", column);

		/* random keyboard presses */
		add_event_to_event_list(this_workload, current_time, KEYPAD_ON, NULL, value, 2, INPUT_EVENT, environment.minimum_operating_speed);

		value = (char**)malloc(sizeof(char*)*2);
		value[0] = (char*)malloc(sizeof(char)*2);
		value[1] = (char*)malloc(sizeof(char)*2);
		sprintf(value[0], "%d", row);
		sprintf(value[1], "%d", column);

		time_to_add = (mpz_t*)malloc(sizeof(mpz_t));
		mpz_init(*time_to_add);
		/* adds keypresses 1us per interrupt ... maybe the file should define this? */
		mpz_add_ui(*time_to_add, *current_time, 45000000); // add 45ms where 50ms to key release assumed maximum keypress rate

		/* random keyboard release */
		add_event_to_event_list(this_workload, time_to_add, KEYPAD_OFF, NULL, value, 2, INPUT_EVENT, environment.minimum_operating_speed);
	}


int num_input_workloads = 0;
workload_t **input_workloads = NULL;

/*---------------------------------------------------------------------------------------------
 * (function: do_default_wave_generation)
 * can't do all the waves at once since runs out of memory...this function is defunct
 *-------------------------------------------------------------------------------------------*/
void do_default_wave_generation()
{
	int i, j;
	void *event_function;
	void (*header_function)(workload_t*, short);
	short header_type = 0;
	int workload_number = -1;
	workload_t *current_workload;

	/* generate the workloads needs to be done */
	if (default_workloads.num_workload_waves > 0)
	{
		for (i = 0; i < default_workloads.num_workload_waves; i++)
		{
			num_input_workloads ++;
			input_workloads = (workload_t**)realloc(input_workloads, sizeof(workload_t*)*num_input_workloads);

			/* add another workload to the global set of input workloads */
			current_workload = generate_wave(default_workloads.workload_waves[i]->benchmark_name, atoi(default_workloads.workload_waves[i]->name));
			input_workloads[num_input_workloads-1] = current_workload;
		}
	}
}

/*---------------------------------------------------------------------------------------------
 * (function: find_workload)
 * Used when we generated all workloads...now defunct
 *-------------------------------------------------------------------------------------------*/
workload_t *find_workload(int workload_number, char *benchmark_name)
{
	int i;

	for (i = 0; i < num_input_workloads; i++)
	{
		if ((strcmp(input_workloads[i]->design_name, benchmark_name) == 0) && (input_workloads[i]->workload_number == workload_number))
		{
			return input_workloads[i];
		}
	}

	return NULL;
}


/*---------------------------------------------------------------------------------------------
 * (function: events_at_same_time)
 *-------------------------------------------------------------------------------------------*/
int events_at_same_time(workload_t *w, int current_index)
{
	int i;
	int count_of_same_time_events = 1; // 1 since the first event that is compared against counts as one
	mpz_t *current_time = w->events.events[current_index]->time;

	/* start comparing times at the current_index and if they're the same then increase the clump of same time events */
	/* we assume that a workload has ordered times, but if we didn't then we'll need to a search */
	for (i = current_index+1; i < w->events.num_events; i++)
	{
		if (mpz_cmp(*current_time, *(w->events.events[i]->time)) == 0)
		{	
			count_of_same_time_events++;
		}
		else
		{
			break;
		}
	}

	return count_of_same_time_events;
}


	LINKED_LIST_NODE *t;
        event_t *traverse;

        /* hook your inputs into their desired inputs */
        for (	t = LinkedListNodePeek(list_of_events);
		t != NULL;
		t = LinkedListNodeNext(t))
	{
		traverse = LinkedListNodeData(t);
	}



/*---------------------------------------------------------------------------------------------
 * (function: generate_all_the_distribution_workloads)
 * Three parameters define data arrival...
 * Data bursts happen every X time_units
 * For a data burst the data is generated every Y time_units
 * Each data burst has data generated for Z time_units
 *
 * So X and Z define something like a step function, and Y defines the frequency of data when
 * the burst is happening
 *-------------------------------------------------------------------------------------------*/
void generate_all_aes_workloads()
{
	int i, j, k;

	int num_X_list_of_time_period_for_data_bursts = 5;
	mpz_t *X_list_of_time_period_for_data_bursts;
	int num_Y_list_of_times_between_data = 9;
	mpz_t *Y_list_of_times_between_data;
	int num_Z_list_of_time_per_data_burst = 5;
	mpz_t *Z_list_of_time_per_data_burst;
	mpz_t total_time;

	FILE *output_file;
	char file_name[4096];
	int file_index_count = 1;

	mpz_t time_min_in_ns;
	mpz_t time_s_in_ns;
	mpz_t time_ms_in_ns;
	mpz_t time_us_in_ns;

	mpz_init_set_str(time_min_in_ns, "60000000000", 10);
	mpz_init_set_str(time_s_in_ns, "1000000000", 10);
	mpz_init_set_str(time_ms_in_ns, "1000000", 10);
	mpz_init_set_str(time_us_in_ns, "1000", 10);

	mpz_init(total_time);
	/* total time = 5 min */
	mpz_mul_ui(total_time, time_min_in_ns, 5);

	X_list_of_time_period_for_data_bursts = (mpz_t*)malloc(sizeof(mpz_t)*num_X_list_of_time_period_for_data_bursts);
	Y_list_of_times_between_data = (mpz_t*)malloc(sizeof(mpz_t)*num_Y_list_of_times_between_data);
	Z_list_of_time_per_data_burst = (mpz_t*)malloc(sizeof(mpz_t)*num_Z_list_of_time_per_data_burst);

	for (i = 0; i < num_Z_list_of_time_per_data_burst; i++)
	{
		mpz_init(Z_list_of_time_per_data_burst[i]);
		mpz_ui_pow_ui(Z_list_of_time_per_data_burst[i], 10, i);  
		mpz_mul(Z_list_of_time_per_data_burst[i], time_ms_in_ns, Z_list_of_time_per_data_burst[i]);
	}

	for (i = 0; i < num_Y_list_of_times_between_data; i++)
	{
		mpz_init(Y_list_of_times_between_data[i]);
		mpz_ui_pow_ui(Y_list_of_times_between_data[i], 10, i);  
		mpz_mul(Y_list_of_times_between_data[i], time_ms_in_ns, Y_list_of_times_between_data[i]);
	}

	for (i = 0; i < num_X_list_of_time_period_for_data_bursts; i++)
	{
		mpz_init(X_list_of_time_period_for_data_bursts[i]);
		mpz_ui_pow_ui(X_list_of_time_period_for_data_bursts[i], 10, i);  
		mpz_mul(X_list_of_time_period_for_data_bursts[i], time_ms_in_ns, X_list_of_time_period_for_data_bursts[i]);
	}

	for (i = 0; i < num_X_list_of_time_period_for_data_bursts; i++)
	{
		for (j = 0; j < num_Y_list_of_times_between_data; j++)
		{
			for (k = 0; k < num_Z_list_of_time_per_data_burst; k++)
			{
				printf("w.%d.GH09.B.2.crypto.xml\n", file_index_count);
				fflush(stdout);
	
				sprintf(file_name, "w.%d.GH09.B.2.crypto.xml", file_index_count);
				file_index_count++;
	
				output_file = fopen(file_name, "w");
				if (output_file == NULL)
				{
					fprintf(stderr, "Can't open filename: %s\n", file_name);
				}
	
				print_aes_header(output_file);
	
				pulse_wave_of_events(output_file, 
						&(X_list_of_time_period_for_data_bursts[i]), 50,
						&(Y_list_of_times_between_data[j]), 50,
						&(Z_list_of_time_per_data_burst[k]), 50,
						&total_time,
						&do_next_aes_event_block_input);
	
				print_aes_tail(output_file);
	
				fclose(output_file);
			}
		}
	}
		
		
}


/*---------------------------------------------------------------------------------------------
 * (function: generate_all_the_glue_logic_workloads)
 *-------------------------------------------------------------------------------------------*/
void generate_all_the_glue_logic_workloads()
{
	int i, j;
	int num_list_of_frequencies = 9;
	mpz_t *list_of_frequencies;
	int num_list_of_bursts_per_second = 5;
	mpz_t *list_of_bursts_per_second; // defined in bursts per minue, 0.006bpm is smallest = 1 bp100ms
	mpz_t *list_of_burst_lengths; // defined in bursts per minue, 0.006bpm is smallest = 1 bp100ms
	FILE *output_file;
	char file_name[4096];
	int file_index_count = 1;

	list_of_frequencies = (mpz_t*)malloc(sizeof(mpz_t)*num_list_of_frequencies);
	list_of_bursts_per_second = (mpz_t*)malloc(sizeof(mpz_t)*num_list_of_bursts_per_second);
	list_of_burst_lengths = (mpz_t*)malloc(sizeof(mpz_t)*num_list_of_bursts_per_second);

	for (i = 0; i < num_list_of_frequencies; i++)
	{
		mpz_init(list_of_frequencies[i]);
		mpz_ui_pow_ui(list_of_frequencies[i], 10, i);  
	}

	for (i = 0; i < num_list_of_bursts_per_second; i++)
	{
		mpz_t time_s_in_ns;
		mpz_init_set_str(time_s_in_ns, "1000000000", 10); // string in nanoseconds for 100 ms

		mpz_init(list_of_bursts_per_second[i]);
		mpz_ui_pow_ui(list_of_bursts_per_second[i], 10, i);  
		/* burst length is 1/10th the time */
		mpz_init(list_of_burst_lengths[i]);
		mpz_cdiv_q(list_of_burst_lengths[i], time_s_in_ns, list_of_bursts_per_second[i]);
		mpz_cdiv_q_ui(list_of_burst_lengths[i], list_of_burst_lengths[i], 10);

		mpz_clear(time_s_in_ns);
	}

	for (i = 0; i < num_list_of_frequencies; i++)
	{
		for (j = 0; j < num_list_of_bursts_per_second; j++)
		{
			if (mpz_cmp(list_of_frequencies[i], list_of_bursts_per_second[j]) < 0)
			{
				continue;
			}

			printf("w.%d.GH09.B.2.glue.xml\n", file_index_count);
			fflush(stdout);

			sprintf(file_name, "w.%d.GH09.B.2.glue.xml", file_index_count);
			file_index_count++;

			output_file = fopen(file_name, "w");
			if (output_file == NULL)
			{
				fprintf(stderr, "Can't open filename: %s\n", file_name);
			}

			print_glue_logic_header(output_file);

			print_glue_logic_event_body(output_file, list_of_frequencies, i, list_of_bursts_per_second, j, list_of_burst_lengths, j);

			print_glue_logic_tail(output_file);

			fclose(output_file);
		}
	}
		
		
}

void print_glue_logic_event_body(FILE *file, mpz_t *frequency, int index1,  mpz_t* bursts_per_s, int index2, mpz_t* length_of_burst, int index3)
{
	int i;
	mpz_t total_time;
	mpz_t time_s_in_ns;
	mpz_t time_per_burst_in_ns;
	mpz_t time_for_frequency;
	mpz_t time_for_this_burst;
	mpz_t current_time;
	int num_signals = 4; 
	char *signals[] = {"A", "B", "C", "I"};
	short is_this_the_first = FALSE;
	char *temp_string;
	char *temp_string2;

	unsigned int percent_of_burst;

	mpz_init_set_str(total_time, "300000000000", 10); // string in nanoseconds for 5 minutes
	mpz_init_set_str(time_s_in_ns, "1000000000", 10); // string in nanoseconds for 100 ms

	mpz_init(time_for_this_burst);
	mpz_init(time_per_burst_in_ns);
	mpz_init(time_for_frequency);
	mpz_init_set_str(current_time, "10000000", 10); // 10 ms

	mpz_cdiv_q(time_per_burst_in_ns, time_s_in_ns, bursts_per_s[index2]);
	mpz_cdiv_q(time_for_frequency, time_s_in_ns, frequency[index1]);
	
	temp_string = mpz_get_str(NULL, 10, frequency[index1]);
	temp_string2 = mpz_get_str(NULL, 10, bursts_per_s[index2]);
	fprintf(file, "\t\t<!-- Freq=%s Num_burst_per_second=%s -->\n", temp_string, temp_string2);
	free(temp_string);
	free(temp_string2);

	while (mpz_cmp(total_time, current_time) > 0)
	{
		/* bursts will last for a guassian for the entire time period */	
		percent_of_burst = rand()%99+1;

		fprintf(file, "\t\t<!-- BURST: %d -->\n", percent_of_burst);

		/* get the time chunk */
		mpz_mul_ui(time_for_this_burst, length_of_burst[index3], 100);	
		mpz_cdiv_q_ui(time_for_this_burst, time_for_this_burst, percent_of_burst);	
		mpz_add(time_for_this_burst, time_for_this_burst, current_time);

		while(mpz_cmp(time_for_this_burst, current_time) > 0)
		{
			is_this_the_first = FALSE;

			for (i = 0; i < num_signals; i++)
			{
				int signal_happens = rand() % 2;

				if (signal_happens == 0)
				{
					if (is_this_the_first == FALSE)
					{

						is_this_the_first = TRUE;

						fprintf(file, "\t\t<event>\n");
						temp_string = mpz_get_str(NULL, 10, current_time);
						fprintf(file, "\t\t\t<time>%sns</time>\n", temp_string);

						free(temp_string);
					}

					/* this signal happens */
					fprintf(file, "\t\t\t<resource>\n");
					fprintf(file, "\t\t\t\t<!-- start with the reset state -->\n");
					if (i == 0)
					{
						fprintf(file, "\t\t\t\t<name>A</name>\n");
					}
					else if (i == 1)
					{
						fprintf(file, "\t\t\t\t<name>B</name>\n");
					}
					else if (i == 2)
					{
						fprintf(file, "\t\t\t\t<name>C</name>\n");
					}
					else if (i == 3)
					{
						fprintf(file, "\t\t\t\t<name>I</name>\n");
					}
					fprintf(file, "\t\t\t\t<value0>%d</value0>\n", rand()%2);
					fprintf(file, "\t\t\t</resource>\n");

				}
			}
			if (is_this_the_first == TRUE)
			{
				fprintf(file, "\t\t</event>\n");
			}
			
			/* decrement the time */
			mpz_add(current_time, current_time, time_for_frequency);
		}

		/* add the time off period */
		mpz_mul_ui(time_for_this_burst, length_of_burst[index3], 100);	
		mpz_cdiv_q_ui(time_for_this_burst, time_for_this_burst, percent_of_burst);	

		mpz_sub(time_for_this_burst, time_per_burst_in_ns, time_for_this_burst);
		mpz_add(current_time, current_time, time_for_this_burst);
	}
}


/*---------------------------------------------------------------------------------------------
 * (function: generate_gpio_workloads)
 *-------------------------------------------------------------------------------------------*/
void generate_gpio_workloads()
{
	generate_bunch_of_gpios("w.1.GH09.B.1.glue.xml", 300);
	generate_bunch_of_gpios("w.2.GH09.B.1.glue.xml", 15000);
	generate_bunch_of_4x4_keypresses("w.3.GH09.B.1.glue.xml");
	generate_bunch_of_8x7_keypresses("w.4.GH09.B.1.glue.xml");
	generate_4x4_throughput_keypresses("w.5.GH09.B.1.glue.xml");
	generate_8x7_throughput_keypresses("w.6.GH09.B.1.glue.xml");
}

/*---------------------------------------------------------------------------------------------
 * (function: generate_bunch_of_gpios)
 * for w.2.B.1
 * Assumes that there are 180000 events in 5m of time which as an event every 1ms
 * 75% of the time this is a pin input change
 * 25% of the time thise is a pin output write
 *-------------------------------------------------------------------------------------------*/
void generate_bunch_of_gpios(char *file_name, long int num_events)
{
	FILE *file;
	long int i;
	int j;
	short eight_gpio_vals[8];

	/* initialize all the input gpio_pins from the external environment */
	for (j = 0; j < 8; j++)
	{
		eight_gpio_vals[j] = rand() % 2;
	}

	/* randomize to my choice of rand seed */
	srand (8);
	/* last 8 bits are outputs
	 * alternating first input bits are interrupted
	 * value is alternating for outputs
	 * no keypad */
	print_gpio_header(file, "101000000000000011111111", "110000000101010100000000", "111000000000000010101010", "100000000000000000000000");

	file = fopen(file_name, "w");
	if (file == NULL)
	{
		fprintf(stderr, "Can't open filename: %s\n", file_name);
	}

		/* now go through all the events and print out there value */
	for (i = 0; i < num_events; i++)
	{
		int event_type = rand()%4;

		fprintf(file, "\t\t<event>\n");
		fprintf(file, "\t\t\t<time>%dms</time>\n", i*(300000/num_events));
		fprintf(file, "\t\t\t<resource>\n");
		if (event_type < 3)
		{
			/* IF a gpio read */
			/* in this case first 8 bits are read bits */
			int pin_num = rand()%8;
			/* toggle value so it's different then before and will cause an interrupt */
			eight_gpio_vals[pin_num] = (eight_gpio_vals[pin_num] + 1) % 2;

			fprintf(file, "\t\t\t\t<name>gpio_pin</name>\n");
			fprintf(file, "\t\t\t\t<value0>%d</value0>\n", pin_num);
			fprintf(file, "\t\t\t\t<value1>%d</value1>\n", eight_gpio_vals[pin_num]);
		}
		else if (event_type == 3)
		{
			/* ELSE IF a write to the value register */
			/* in this case last 8 bits are write bits */
			int val_to_write = rand();
			char *temp_string = convert_int_to_bit_string(val_to_write, 8);

			fprintf(file, "\t\t\t\t<name>spi</name>\n");
			fprintf(file, "\t\t\t\t<value0>1110000000000000%s</value0>\n", temp_string);

			free(temp_string);
		}
		fprintf(file, "\t\t\t</resource>\n");
		fprintf(file, "\t\t</event>\n");
	}

	print_gpio_tail(file);

	fclose(file);
}

/*---------------------------------------------------------------------------------------------
 * (function: generate_bunch_of_RxC_keypresses)
 * for w.5.B.1 column_size = 7 row_size = 8
 *
 * 30 possible bursts over 5 mins (1 every 6s)
 * possible of 18 keypresses over the burst
 * time betwen keypresses is 50ms - 100ms
 *-------------------------------------------------------------------------------------------*/
void generate_bunch_of_8x7_keypresses(char *file_name)
{
	FILE *file;
	int i, j;
	long int num_bursts = 30;
	/* way to change the keypad size */
	int column_size = 7;
	int row_size = 8;

	/* randomize to my choice of rand seed */
	srand (8);

	file = fopen(file_name, "w");
	if (file == NULL)
	{
		fprintf(stderr, "Can't open filename: %s\n", file_name);
	}
	/* no gpios
	 * no interrupts
	 * no value
	 * 8x7 keypad */
	print_gpio_header(file, "101000000000000000000000", "110000000000000000000000", "111000000000000000000000", "100000000001111100000000");

		/* now go through all the events and print out there value */
	for (i = 0; i < num_bursts; i++)
	{
		int event_type = rand()%100;
		long int time;
		long int old_time;

		if (event_type < 20)
		{
			/* 10% chance that a burst happens */
			int num_presses = rand()%18; // 1s of keypressing at 20keypresses per second

			/* 10 ms + burst * 6seconds + press * 50ms */ 
			time = 10 + (i * 6000) + (0 * 50);
			/* must have an output list that is in time order */
			assert(old_time <= time);

			for (j = 0; j < num_presses; j++)
			{
				/* in this case first 8 bits are read bits */
				int row = rand()%row_size;
				int column = rand()%column_size;
				
				/* add some variance between keypresses */
				time += 50+rand()%50;

				fprintf(file, "\t\t<event>\n");
				fprintf(file, "\t\t\t<time>%dms</time>\n", time);
				fprintf(file, "\t\t\t<resource>\n");

				fprintf(file, "\t\t\t\t<name>keypad</name>\n");
				fprintf(file, "\t\t\t\t<value0>%d</value0>\n", row);
				fprintf(file, "\t\t\t\t<value1>%d</value1>\n", column);

				fprintf(file, "\t\t\t</resource>\n");
				fprintf(file, "\t\t</event>\n");
			}
			old_time = time;
		}
	}

	print_gpio_tail(file);

	fclose(file);
}

/*---------------------------------------------------------------------------------------------
 * (function: generate_bunch_of_RxC_keypresses)
 * for w.3.B.1 column_size = 4 row_size = 4
 *
 * 30 possible bursts over 5 mins (1 every 6s)
 * possible of 18 keypresses over the burst
 * time betwen keypresses is 50ms - 100ms
 *-------------------------------------------------------------------------------------------*/
void generate_bunch_of_4x4_keypresses(char *file_name)
{
	FILE *file;
	int i, j;
	long int num_bursts = 30;
	/* way to change the keypad size */
	int column_size = 4;
	int row_size = 4;

	/* randomize to my choice of rand seed */
	srand (8);

	file = fopen(file_name, "w");
	if (file == NULL)
	{
		fprintf(stderr, "Can't open filename: %s\n", file_name);
	}

	/* no gpios
	 * no interrupts
	 * no value
	 * 4x4 keypad */
	print_gpio_header(file, "101000000000000000000000", "110000000000000000000000", "111000000000000000000000", "100000000010001100000000");

	/* now go through all the events and print out there value */
	for (i = 0; i < num_bursts; i++)
	{
		int event_type = rand()%100;
		long int time;
		long int old_time;

		if (event_type < 20)
		{
			/* 10% chance that a burst happens */
			int num_presses = rand()%18; // 1s of keypressing at 20keypresses per second

			/* 10 ms + burst * 6seconds + press * 50ms */ 
			time = 10 + (i * 6000) + (0 * 50);
			/* must have an output list that is in time order */
			assert(old_time <= time);

			for (j = 0; j < num_presses; j++)
			{
				/* in this case first 8 bits are read bits */
				int row = rand()%row_size;
				int column = rand()%column_size;
				
				/* add some variance between keypresses */
				time += 50+rand()%50;

				fprintf(file, "\t\t<event>\n");
				fprintf(file, "\t\t\t<time>%dms</time>\n", time);
				fprintf(file, "\t\t\t<resource>\n");

				fprintf(file, "\t\t\t\t<name>keypad</name>\n");
				fprintf(file, "\t\t\t\t<value0>%d</value0>\n", row);
				fprintf(file, "\t\t\t\t<value1>%d</value1>\n", column);

				fprintf(file, "\t\t\t</resource>\n");
				fprintf(file, "\t\t</event>\n");
			}
			old_time = time;
		}
	}

	print_gpio_tail(file);

	fclose(file);
}

/*---------------------------------------------------------------------------------------------
 * (function: generate_RxC_throughput_keypresses)
 * for w.6.B.1 column_size = 7 row_size = 8
 *
 * 3600 events = 5 minutes / 50ms per type ... we might not get this many since time between
 * presses can vary and thie is a 2% chance of a pause
 *-------------------------------------------------------------------------------------------*/
void generate_8x7_throughput_keypresses(char *file_name)
{
	FILE *file;
	int i, j;
	int num_events = 3600; // 180000ms / 50ms(per type) 
	long int time = 10;
	int column_size = 7;
	int row_size = 8;

	/* randomize to my choice of rand seed */
	srand (8);

	file = fopen(file_name, "w");
	if (file == NULL)
	{
		fprintf(stderr, "Can't open filename: %s\n", file_name);
	}

	/* no gpios
	 * no interrupts
	 * no value
	 * 8x7 keypad */
	print_gpio_header(file, "101000000000000000000000", "110000000000000000000000", "111000000000000000000000", "100000000001111100000000");

	/* now go through all the events and print out there value */
	for (i = 0; i < num_events; i++)
	{
		/* 10 ms + burst * 6seconds + press * 50ms */ 
		/* add some variance between keypresses */
		time += rand()%100 < 3 ? 50+rand()%1000 : 50+rand()%50;

		if (time < 180000)
		{
			/* in this case first 8 bits are read bits */
			int row = rand()%row_size;
			int column = rand()%column_size;
				
			fprintf(file, "\t\t<event>\n");
			fprintf(file, "\t\t\t<time>%dms</time>\n", time);
			fprintf(file, "\t\t\t<resource>\n");

			fprintf(file, "\t\t\t\t<name>keypad</name>\n");
			fprintf(file, "\t\t\t\t<value0>%d</value0>\n", row);
			fprintf(file, "\t\t\t\t<value1>%d</value1>\n", column);

			fprintf(file, "\t\t\t</resource>\n");
			fprintf(file, "\t\t</event>\n");
		}
		else
		{
			break;
		}
	}

	print_gpio_tail(file);

	fclose(file);
}

/*---------------------------------------------------------------------------------------------
 * (function: generate_RxC_throughput_keypresses)
 * for w.4.B.1 column_size = 4 row_size = 4
 *
 * 3600 events = 5 minutes / 50ms per type ... we might not get this many since time between
 * presses can vary and thie is a 2% chance of a pause
 *-------------------------------------------------------------------------------------------*/
void generate_4x4_throughput_keypresses(char *file_name)
{
	FILE *file;
	int i, j;
	int num_events = 3600; // 180000ms / 50ms(per type) 
	long int time = 10;
	int column_size = 7;
	int row_size = 8;

	/* randomize to my choice of rand seed */
	srand (8);

	file = fopen(file_name, "w");
	if (file == NULL)
	{
		fprintf(stderr, "Can't open filename: %s\n", file_name);
	}

	/* no gpios
	 * no interrupts
	 * no value
	 * 4x4 keypad */
	print_gpio_header(file, "101000000000000000000000", "110000000000000000000000", "111000000000000000000000", "100000000010001100000000");

	/* now go through all the events and print out there value */
	for (i = 0; i < num_events; i++)
	{
		/* 10 ms + burst * 6seconds + press * 50ms */ 
		/* add some variance between keypresses */
		time += rand()%100 < 3 ? 50+rand()%1000 : 50+rand()%50;

		if (time < 180000)
		{
			/* in this case first 8 bits are read bits */
			int row = rand()%row_size;
			int column = rand()%column_size;
				
			fprintf(file, "\t\t<event>\n");
			fprintf(file, "\t\t\t<time>%dms</time>\n", time);
			fprintf(file, "\t\t\t<resource>\n");

			fprintf(file, "\t\t\t\t<name>keypad</name>\n");
			fprintf(file, "\t\t\t\t<value0>%d</value0>\n", row);
			fprintf(file, "\t\t\t\t<value1>%d</value1>\n", column);

			fprintf(file, "\t\t\t</resource>\n");
			fprintf(file, "\t\t</event>\n");
		}
		else
		{
			break;
		}
	}

	print_gpio_tail(file);

	fclose(file);
}

/*---------------------------------------------------------------------------------------------
 * (function: do_next_glue_logic_input)
 *-------------------------------------------------------------------------------------------*/
void do_next_glue_logic_input(mpz_t *current_time, workload_t* this_workload, short first_time)
{
	int i;
	char *temp_string;
	int num_signals = 4; 
	char **value;

	/* go through each signal and randomly generate the signal or not */
	for (i = 0; i < num_signals; i++)
	{
		int signal_happens = my_rand() % num_signals;

		if (signal_happens == 0)
		{
			value = (char**)malloc(sizeof(char*)*1);
			value[0] = (char*)malloc(sizeof(char)*2);

			sprintf(value[0], "%d", my_rand()%2);

			if (i == 0)
			{
				add_event_to_event_list(this_workload, current_time, A_in, NULL, value, 1, INPUT_EVENT);
			}
			else if (i == 1)
			{
				add_event_to_event_list(this_workload, current_time, B_in, NULL, value, 1, INPUT_EVENT);
			}
			else if (i == 2)
			{
				add_event_to_event_list(this_workload, current_time, C_in, NULL, value, 1, INPUT_EVENT);
			}
			else if (i == 3)
			{
				add_event_to_event_list(this_workload, current_time, I_in, NULL, value, 1, INPUT_EVENT);
			}
		}
	}
}

/*---------------------------------------------------------------------------------------------
 * (function: gl_process_event_clump)
 * from old glu logic
 *-------------------------------------------------------------------------------------------*/
void gl_process_event_clump(LINKED_LIST *output_timeline, gl_device_state_t *state, event_t *clump_to_process)
{
	int i, j; 
	char *resource_name;
	mpz_t *output_time = (mpz_t*)malloc(sizeof(mpz_t));
	short output_time_used = FALSE;
	char *timeX10ms;
	mpz_t time10ms;
	char *timeX90ms;
	mpz_t time90ms;
	short time10ms_used = FALSE;
	short time90ms_used = FALSE;

	/* setup the timeX character strings */
	timeX10ms = (char*)malloc(sizeof(char)*11);
	sprintf(timeX10ms, "timeX10ms");
	mpz_init_set_str(time10ms, "10000000", 10); // string in nanoseconds
	timeX90ms = (char*)malloc(sizeof(char)*11);
	sprintf(timeX90ms, "timeX90ms");
	mpz_init_set_str(time90ms, "90000000", 10); // string in nanoseconds

	/* setup the output time delay ... this probably should have details of all the signals in later versions */
	mpz_init(*output_time);
	mpz_set(*output_time, *(clump_to_process->time));
	mpz_add_ui(*output_time, *output_time, (unsigned int)13000); // add 13us ... this is a hard limit on Y

	/* now go through all the events in the current clump which represents a timestep */
	for (j = 0; j < clump_to_process->num_resources; j++)
	{	
		char *name = clump_to_process->resources[j]->name;
		int num_values = clump_to_process->resources[j]->num_values;
		char **value = clump_to_process->resources[j]->value;
		short to_free;

		/* need to get theresource name for dfault names */
		if (clump_to_process->resources[j]->name != NULL)
		{
			name = clump_to_process->resources[j]->name;
			to_free = FALSE;
		}
		else
		{
			name = get_default_resource_name(clump_to_process->resources[j]->default_name);
			to_free = TRUE;
		}

		/* ---------------------------------------------------------------------------------------------------------------
		 * This is the unique part for handling each resource type
		 * -------------------------------------------------------------------------------------------------------------*/
		if (strcmp(name, "reset") == 0)
		{
			if (strcmp(value[0], "1") == 0)
			{
				state->X = ZERO;
				state->O = bitZ;
				state->Y = ZERO;
				state->A = bitX;
				state->B = bitX;
				state->C = bitX;
				state->I = bitX;
			}
			else if (strcmp(value[0], "0") == 0)
			{
				/* insert a time event for X to change value in 10ms */
				mpz_t *new_time;
				new_time = (mpz_t*)malloc(sizeof(mpz_t));

				mpz_init(*new_time);
				mpz_add(*new_time, *(clump_to_process->time), time10ms);

				/* add the event to the list in its proper spot...can't happen before this output */
				/* this is an internal timer that tells us when X will fire */
				add_event_to_timeline(output_timeline, new_time, timeX10ms,  NULL, 0, TIMING_EVENT); 

				state->new_X = ONE;
				time10ms_used = TRUE;
			}
		}
		else if (strcmp(name, "timeX10ms") == 0)
		{
			/* IF - a timer for X fires after 10ms put a 90ms timer on */
			mpz_t *new_time;
			new_time = (mpz_t*)malloc(sizeof(mpz_t));

			mpz_init(*new_time);
			mpz_add(*new_time, *(clump_to_process->time), time90ms);

			/* add the event to the list in its proper spot...can't happen before this output */
			add_event_to_timeline(output_timeline, new_time, timeX90ms,  NULL, 0, TIMING_EVENT); 

			/* update the X value */
			state->new_X = ZERO;
			time90ms_used = TRUE;
		}
		else if (strcmp(name, "timeX90ms") == 0)
		{
			/* IF - a timer for X fires */
			mpz_t *new_time;
			new_time = (mpz_t*)malloc(sizeof(mpz_t));

			mpz_init(*new_time);
			mpz_add(*new_time, *(clump_to_process->time), time10ms);

			/* add the event to the list in its proper spot...can't happen before this output */
			add_event_to_timeline(output_timeline, new_time, timeX10ms,  NULL, 0, TIMING_EVENT); 

			/* update the X value */
			state->new_X = ONE;
			time10ms_used = TRUE;
		}
		else if (strcmp(name, "A") == 0)
		{
			/* IF - a input */
			state->A = atoi(value[0]);
		}
		else if (strcmp(name, "B") == 0)
		{
			/* IF - a input */
			state->B = atoi(value[0]);
		}
		else if (strcmp(name, "C") == 0)
		{
			/* IF - a input */
			state->C = atoi(value[0]);
		}
		else if (strcmp(name, "I") == 0)
		{
			/* IF - a input */
			state->I = atoi(value[0]);
		}

		if (to_free == TRUE)
		{
			free(name);
		}
	}

	/* cleanup the clump */
	free_event_clump(clump_to_process);

	/* on any event check if an output vector needs to be updated */
	output_time_used = send_outputs(state, output_time, output_timeline, output_time_used);

	if (output_time_used == FALSE)
	{
		/* free the time if it wasn't used ... can happen since some inputs only change internal data and not external outputs */
		mpz_clear(*output_time);
		free(output_time);
	}

	if (time90ms_used == FALSE)
	{
		free(timeX90ms);
	}
	if (time10ms_used == FALSE)
	{
		free(timeX10ms);
	}

	mpz_clear(time10ms);
	mpz_clear(time90ms);
}

/*---------------------------------------------------------------------------------------------
 * (function: check_for_gpio_output_updates)
 * Simulates all the logic inside the design
 *-------------------------------------------------------------------------------------------*/
short send_outputs(gl_device_state_t *state, mpz_t *output_time, LINKED_LIST *output_timeline, short output_time_used)
{
	char **char_values;
	char *resource_name;
	short return_output_time_used = output_time_used;
	short new_Y;
	short new_O;

	/* Y */
	if (state->A < 2 && state->B < 2 && state->C < 2)
	{
		/* make sure all the inputs are in a non X,Z state */
		new_Y = (state->A || state->B) && !(state->C);
	}
	else
	{
		new_Y = bitX;
	}

	if (new_Y != state->Y)
	{
		state->Y = new_Y;

		/* create the resource name */
		resource_name = (char*)malloc(sizeof(char)*2);
		sprintf(resource_name, "Y");
	
		char_values = (char **)malloc(sizeof(char*)*1);
		char_values[0] = (char *)malloc(sizeof(char)*2);

		/* create the string with the bit */
		bit_to_string(char_values[0], state->Y);
	
		/* add this event to the output list since this results is an output event */
		add_event_to_timeline(output_timeline, output_time, resource_name, char_values, 1, OUTPUT_EVENT); 
		return_output_time_used = TRUE;
	
		free(char_values);
	}

	/* X - analyse X before O becuase of the dependence */
	if (state->new_X != state->X)
	{
		state->X = state->new_X;

		/* create the resource name */
		resource_name = (char*)malloc(sizeof(char)*2);
		sprintf(resource_name, "X");
	
		char_values = (char **)malloc(sizeof(char*)*1);
		char_values[0] = (char *)malloc(sizeof(char)*2);

		/* create the string with the bit */
		bit_to_string(char_values[0], state->X);
	
		/* add this event to the output list since this results is an output event */
		add_event_to_timeline(output_timeline, output_time, resource_name, char_values, 1, OUTPUT_EVENT); 
		return_output_time_used = TRUE;
	
		free(char_values);
	}

	/* O */
	if (state->I < 2)
	{
		/* make sure all the inputs are in a non X,Z state */
		if (state->X == ONE)
		{
			new_O = state->I;
		}
		else
		{
			new_O = bitZ;
		}
	}
	else
	{
		new_O = bitZ;
	}

	if (new_O != state->O)
	{
		state->O = new_O;

		/* create the resource name */
		resource_name = (char*)malloc(sizeof(char)*2);
		sprintf(resource_name, "O");
	
		char_values = (char **)malloc(sizeof(char*)*1);
		char_values[0] = (char *)malloc(sizeof(char)*2);

		/* create the string with the bit */
		bit_to_string(char_values[0], state->O);
	
		/* add this event to the output list since this results is an output event */
		add_event_to_timeline(output_timeline, output_time, resource_name, char_values, 1, OUTPUT_EVENT); 
		return_output_time_used = TRUE;
	
		free(char_values);
	}

	return return_output_time_used;
}

/* snippet of code from lziv simulation for char and hex switching */
if (strcmp(value[1], "char") == 0)
			{
				state->num_input_block_data = strlen(value[0]);
				state->input_block_data = (bit_vector **)malloc(sizeof(bit_vector*)*state->num_input_block_data);

				for (i = 0; i < state->num_input_block_data; i++)
				{
					state->input_block_data[i] = new_bit_vector(8);
					set_bits_with_short_at(state->input_block_data[i], 0, value[0][i]);
				}
			}
			else if (strcmp(value[1], "hex") == 0)
