/* 
 * Copyright (c) 2008 - Peter Jamieson
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "types.h"
#include "globals.h"
#include "util.h"
#include "gmp.h"
#include "read_xml_config_file.h"

environment_t environment;

void read_top_environment(xmlNode * a_node);
void read_the_environment(xmlNode * a_node, environment_t *environment);

/*---------------------------------------------------------------------------------------------
 * (function: read_environment)
 *-------------------------------------------------------------------------------------------*/
void read_environment(char *file_name)
{
	xmlDoc *doc = NULL;
	xmlNode *root_element = NULL;

	/* this initialize the library and check potential ABI mismatches between the version it was compiled for and the actual shared library used. */
	LIBXML_TEST_VERSION

	/*parse the file and get the DOM */
	doc = xmlParseFile(file_name);

	if (doc == NULL) 
	{
		printf("error: could not parse file LIB file\n");
	}

	/*Get the root element node */
	root_element = xmlDocGetRootElement(doc);

	/* parse in the XML to data structs */
	read_top_environment(root_element);

	/*free the document */
	xmlFreeDoc(doc);

	/* Free the global variables that may have been allocated by the parser.  */
	xmlCleanupParser();
}

/*---------------------------------------------------------------------------------------------
 * (function: read_top_environment)
 *-------------------------------------------------------------------------------------------*/
void read_top_environment(xmlNode * a_node)
{
	int i, j;
	xmlNode *top_node = NULL;
	xmlChar *number_tfe_string = NULL;

	for (top_node = a_node; top_node; top_node = top_node->next) 
	{
		if (strcmp(top_node->name, "environment") == 0)
		{
			read_the_environment(top_node->children, &environment);
		}
	}
}

/*---------------------------------------------------------------------------------------------
 * (function: read_the_environment)
 *-------------------------------------------------------------------------------------------*/
void read_the_environment(xmlNode * a_node, environment_t *environment)
{
	xmlNode *cur_node = NULL;

	for (cur_node = a_node; cur_node; cur_node = cur_node->next) 
	{
		if (strcmp(cur_node->name, "minimum_operating_speed") == 0)
		{
			char *temp_string = get_text(cur_node->children);
			environment->minimum_operating_speed = time_to_nanoseconds(temp_string);
			free(temp_string);
		}
		else if (strcmp(cur_node->name, "minimum_sampling_speed") == 0)
		{
			char *temp_string = get_text(cur_node->children);
			environment->minimum_sampling_speed = time_to_nanoseconds(temp_string);
			free(temp_string);
		}
		else if (strcmp(cur_node->name, "minimum_arrival_rate_on_serial_interfaces") == 0)
		{
			char *temp_string = get_text(cur_node->children);
			environment->minimum_arrival_rate_on_serial_interfaces = time_to_nanoseconds(temp_string);
			free(temp_string);
		}
		else if (strcmp(cur_node->name, "minimum_arrival_rate_on_parallel_interfaces") == 0)
		{
			char *temp_string = get_text(cur_node->children);
			environment->minimum_arrival_rate_on_parallel_interfaces = time_to_nanoseconds(temp_string);
			free(temp_string);
		}
	}
}
