/* 
 * Copyright (c) 2008 - Peter Jamieson
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * */
#include "gmp.h"
#include "bitvector.h"

#define assert_t struct assert_t_t
#define resource_t struct resource_t_t
#define workload_t struct workload_t_t
#define initialization_t struct initialization_t_t
#define condition_t struct condition_t_t
#define events_t struct events_t_t
#define event_t struct event_t_t

#define environment_t struct environment_t_t

#define workload_waves_t struct workload_waves_t_t
#define waves_t struct waves_t_t

#define workload_gen_t struct workload_gen_t_t
#define config_t struct config_t_t
#define golden_func_t struct golden_func_t_t
#define workload_description_t struct workload_description_t

#ifndef TYPES_H
#define TYPES_H

#define assert(x) {if(!(x)){__asm("int3");}} // causes an interrupt in GDB
// bitvector library (PETER_LIB) defines it, so we don't
//#ifndef TRUE      /* Some compilers predefine TRUE, FALSE */
//typedef enum {FALSE, TRUE} boolean;
//#else
//typedef int boolean;
//#endif

/* This is the OUTPUT type for workloads */
#define XML 1
#define VERILOG 2

/* ----------------------------------------------- */

environment_t
{
	mpz_t* minimum_operating_speed; // fast clock in ns
	mpz_t* minimum_sampling_speed; // heart beat clock in ns
	mpz_t* minimum_arrival_rate_on_serial_interfaces;
	mpz_t* minimum_arrival_rate_on_parallel_interfaces;
};

/* ----------------------------------------------- */

workload_waves_t
{
	char *name;
	char *design_name;
	mpz_t *X_ns;
	mpz_t *Y_ns;
	mpz_t *Z_ns;
	mpz_t *total_time;
	short X_deviation;
	short Y_deviation;
	short Z_deviation;
	int seed;
	int *workload_value; // these include the mode of the design and the markov percentages...all have to be numbers
	int num_workload_value;
	char **workload_string; 
	int num_workload_string;
};
/* The parameters of a specific waveform.  See  void pulse_wave_of_events(...) for some of the details
 * of what the parameters mean */

waves_t
{
	int num_workload_waves;
	workload_waves_t **workload_waves;
};
/* High level holder of all the waves */

/* ----------------------------------------------- */

workload_description_t
{
	char *design_name;
	int *workloads;
	int num_workloads;

	int num_workload_file_names;
	char **workload_file_names;
};

golden_func_t
{
	short output_type;

	int num_benchmarks;
	workload_description_t **benchmarks;
};

workload_gen_t
{
	short output_type;

	int num_benchmarks;
	workload_description_t **benchmarks;
};

config_t
{
	workload_gen_t *wg;
	golden_func_t *gf;
};
/* Top level of the config structure.  It contains information about the workload generation and the
 * golden functional model.  Both of these sub structures describe which workloads and benchmarks to 
 * exercise on */

/* ----------------------------------------------- */

// Different event types on the timeline
#define OUTPUT_EVENT 0
#define INPUT_EVENT 1
#define COND_EVENT 2
#define TIMING_EVENT 3

// TRUE and FALSE defined in PETER_LIB
#define ZERO 0
#define ONE 1
#define bitX 2
#define bitZ 3

// for reducing memory with default names.  These are defaults for resource names instead of the string 
#define SPI 0
#define GPIO_PIN 1
#define INPUT 3
#define INPUT_BLOCK 4
#define INPUT_DATA_VALID 5
#define BUS 6
#define IDLE 7
#define KEYPAD_ON 8
#define KEYPAD_OFF 9

resource_t
{
	short default_name;
	char *name;
	char **value;
	int num_values;
};

assert_t
{
	mpz_t *time;

	int num_resources;
	resource_t **resources;
};

events_t
{
	event_t **events;
	int num_events;
};

event_t
{
	short type;

	mpz_t *time;

	int num_resources;
	resource_t **resources;

	int num_assertions;
	assert_t **assertions;
};

condition_t
{
	int num_if_resource_pairs;
	resource_t **ifs;
	resource_t **then_resources;

	mpz_t **then_times;
};

workload_t
{
	condition_t *conditions;
	events_t *events;
	char *design_name;
	int workload_number;
};
/* the workload structure is the top level of the events and conditions that make up either a workload
 * or a simulation of a workload.  The main structure within the workload is the events (a timeline of events) */

/* ----------------------------------------------- */
#endif
