/* 
 * Copyright (c) 2008 - Peter Jamieson
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * */
#include <stdlib.h>
#include "globals.h"
#include "types.h"
#include "gmp.h"
#include "util.h"
#include "wave_generation.h"
#include "wg_2dconv.h"
#include "fm_2dconv.h"

// 400 x 400 matrix + 5 x 5 matrix
#define TDCONV_BLOCK_SIZE 160000+25

/*---------------------------------------------------------------------------------------------
 * (function: do_next_2dconv_event_block_input)
 * Here we create a big block of random data, and then send input valid signals that
 * reads in the data from the initial block.
 *-------------------------------------------------------------------------------------------*/
void do_next_2dconv_event_block_input(mpz_t *current_time, workload_t *this_workload, workload_waves_t *wave, short first_time)
{
	int i;
	char *temp_string;
	static short is_new_block = TRUE;
	static short is_convolution_matrix = TRUE;
	static num_blocks = 0;
	int block_size = MATRIX_SIZE*MATRIX_SIZE; 
		
	char *resource_name;
	char **value;

	if (first_time == TRUE)
	{
		is_new_block = TRUE;
		num_blocks = 0;
		is_convolution_matrix = TRUE;
	}

	if ((is_new_block == TRUE) || (num_blocks == 0))
	{
		/* IF we need to make new block that will be read in per data_valid */
		value = (char**)malloc(sizeof(char*)*1);
		value[0] = (char*)malloc(sizeof(char)*(block_size*4+CONVOLUTION_MATRIX_SIZE*CONVOLUTION_MATRIX_SIZE*4+1));
		value[0][0] = '\0';

		/* make an input matrix defined by block_size * 4 since 4hex = 16 bits */
		temp_string = create_hex_string(block_size*4);
#ifdef INT_CALC
		/* this defines the kernel filter that is passed in to the core.  This is if we have an integer core ... FOR DEBUGGING */
		sprintf(value[0], "000100010001000100010001000100010001000100010001ffe7000100010001000100010001000100010001000100010001%s", temp_string);
#endif
#ifdef FIXEDPOINT_CALC
		/* this defines the kernel filter that is passed in to the core.  This is if we have a fixed point core.
		 * we're passing in [[1,1,1,1,1],[1,1,1,1,1],[1,1,-25,1,1],[1,1,1,1,1],[1,1,1,1,1] */
		sprintf(value[0], "010001000100010001000100010001000100010001000100e700010001000100010001000100010001000100010001000100%s", temp_string);
#endif
		free(temp_string);

		add_event_to_event_list(this_workload, current_time, INPUT_BLOCK, NULL, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

		/*  since each hex value is 4 bits and input is 16 bits, then multiply by 4 */
		num_blocks = block_size * 4 + CONVOLUTION_MATRIX_SIZE*CONVOLUTION_MATRIX_SIZE*4;
		is_new_block = FALSE;

		is_convolution_matrix = TRUE;
	}
	else if (num_blocks > 0)
	{
		/* ELSE we need to send the data valid signal to request new block to be read */
		value = (char**)malloc(sizeof(char*)*1);
		value[0] = (char*)malloc(sizeof(char)*2);
		sprintf(value[0], "1");

		if (num_blocks == 0)
		{
			/* IF - input matrix so make sure that the matrix signal is on */
			resource_name = (char*)malloc(sizeof(char)*19);
			sprintf(resource_name, "convolution_matrix");
			add_event_to_event_list(this_workload, current_time, -1, resource_name, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

			value = (char**)malloc(sizeof(char*)*1);
			value[0] = (char*)malloc(sizeof(char)*2);
			sprintf(value[0], "1");
			add_event_to_event_list(this_workload, current_time, INPUT_DATA_VALID, NULL, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);
		}
		else if (num_blocks == CONVOLUTION_MATRIX_SIZE*CONVOLUTION_MATRIX_SIZE-1)
		{
			/* ELSE IF - last signal in the input matrix so turn off signal to indicate now it's the convolution matrix coming in */
			add_event_to_event_list(this_workload, current_time, INPUT_DATA_VALID, NULL, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

			value = (char**)malloc(sizeof(char*)*1);
			value[0] = (char*)malloc(sizeof(char)*2);
			sprintf(value[0], "0");

			resource_name = (char*)malloc(sizeof(char)*19);
			sprintf(resource_name, "convolution_matrix");
			add_event_to_event_list(this_workload, current_time, -1, resource_name, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);
		}
		else
		{
			/* ELSE - normal matrix arrival situation that is processed per event */
			add_event_to_event_list(this_workload, current_time, INPUT_DATA_VALID, NULL, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);
		}

		num_blocks --;
	}
}
