/* 
 * Copyright (c) 2008 - Peter Jamieson
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * */
#include <stdlib.h>
#include "globals.h"
#include "types.h"
#include "gmp.h"
#include "util.h"
#include "wave_generation.h"
#include "wg_bridge.h"
#include "fm_bridge.h"

/*---------------------------------------------------------------------------------------------
 * (function: do_next_bridge_event_block_input)
 * This function generates either a write or a read
 * 
 * Assumption is we never read something that hasn't been written yet, so the first
 * phase does a bunch of writes
 *-------------------------------------------------------------------------------------------*/
void do_next_bridge_event_input(mpz_t *current_time, workload_t *this_workload, workload_waves_t *wave, short first_time)
{
	int i;	
	char **value;
	char *resource_name;
	int read_or_write = my_rand() % 100;
	int address = my_rand() % BRIDGE_ADDRESS_RANGE;
	int write_value = my_rand() % VALUE_RANGE;
	static int first_writes = 0;
	static mpz_t time_to_wait_for_last_event;
	
	assert(wave->num_workload_value == 1);

	if (first_time == TRUE)
	{
		first_writes = 0;
		mpz_init(time_to_wait_for_last_event);
	}

	if (mpz_cmp(*(current_time), time_to_wait_for_last_event) < 0)
	{
		/* this is the case when a signal needs to complete its time before safe, and we need to wait */
		return;
	}

	if (first_writes < BRIDGE_ADDRESS_RANGE)
	{
		/* initialize the memory spots with a value = to the address in the initial steps */
		value = (char**)malloc(sizeof(char*)*2);
		value[0] = (char*)malloc(sizeof(char)*4);
		sprintf(value[0], "%d", first_writes);
		value[1] = (char*)malloc(sizeof(char)*4);
		sprintf(value[1], "%d", first_writes);

		resource_name = (char*)malloc(sizeof(char)*6);
		sprintf(resource_name, "write");

		add_event_to_event_list(this_workload, current_time, -1, resource_name, value, 2, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

		/* create the time when this event should finish by */
		/* 24 bits at the serial clock rate + a little bit of time */
		mpz_mul_ui(time_to_wait_for_last_event, *(environment.minimum_arrival_rate_on_serial_interfaces), 24);
		mpz_add(time_to_wait_for_last_event, *(current_time), time_to_wait_for_last_event); 

		first_writes ++;
	}
	else
	{
		/* once we've initialized the memory we go into random behaviour */
		if (read_or_write < wave->workload_value[0])
		{
			/* the master on the parallel bus asks for a read */
			value = (char**)malloc(sizeof(char*)*1);
			value[0] = (char*)malloc(sizeof(char)*4);
			sprintf(value[0], "%d", address);

			resource_name = (char*)malloc(sizeof(char)*5);
			sprintf(resource_name, "read");

			add_event_to_event_list(this_workload, current_time, -1, resource_name, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

			/* create the time when this event should finish by */
			/* 24 bits read at the serial clock rate + a little bit of time */
			mpz_mul_ui(time_to_wait_for_last_event, *(environment.minimum_arrival_rate_on_serial_interfaces), 24);
			mpz_add(time_to_wait_for_last_event, *(current_time), time_to_wait_for_last_event); 
		}
		else if (read_or_write >= wave->workload_value[0])
		{
			/* the master on the parallel bus does a write */
			value = (char**)malloc(sizeof(char*)*2);
			value[0] = (char*)malloc(sizeof(char)*4);
			sprintf(value[0], "%d", address);
			value[1] = (char*)malloc(sizeof(char)*7);
			sprintf(value[1], "%d", write_value);

			resource_name = (char*)malloc(sizeof(char)*6);
			sprintf(resource_name, "write");

			add_event_to_event_list(this_workload, current_time, -1, resource_name, value, 2, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

			/* create the time when this event should finish by */
			/* 24 bits read at the serial clock rate + a little bit of time */
			mpz_mul_ui(time_to_wait_for_last_event, *(environment.minimum_arrival_rate_on_serial_interfaces), 24);
			mpz_add(time_to_wait_for_last_event, *(current_time), time_to_wait_for_last_event); 
		}
	}
}
