/* 
 * Copyright (c) 2008 - Peter Jamieson
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * */
#include <stdlib.h>
#include "globals.h"
#include "types.h"
#include "gmp.h"
#include "util.h"
#include "wave_generation.h"
#include "wg_lziv.h"
#include "fm_lziv.h"

/*---------------------------------------------------------------------------------------------
 * (function: do_next_lziv_event_block_input)
 * Creates a random block of data that is sent in blocks by an input valid signal
 *-------------------------------------------------------------------------------------------*/
void do_next_lziv_event_block_input(mpz_t *current_time, workload_t *this_workload, workload_waves_t *wave, short first_time)
{
	int i;
	char *temp_string;
	static short is_new_block = TRUE;
	static num_blocks = 0;
		
	char *resource_name;
	char **value;

	if (first_time == TRUE)
	{
		is_new_block = TRUE;
		num_blocks = 0;
	}

	if (is_new_block == TRUE)
	{
		/* IF we need to make new block that will be read in per data_valid */
		value = (char**)malloc(sizeof(char*)*1);
		value[0] = (char*)malloc(sizeof(char)*(wave->workload_value[0]+1));
		value[0][0] = '\0';

		temp_string = create_hex_string(wave->workload_value[0]);
		sprintf(value[0], "%s%s", value[0], temp_string);
		free(temp_string);

		add_event_to_event_list(this_workload, current_time, INPUT_BLOCK, NULL, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

		/*  since each hex value is 4 bits and input is 8 bits, then divide by 2 */
		num_blocks = wave->workload_value[0]/2;
		is_new_block = FALSE;
	}
	else if (num_blocks > 0)
	{
		/* ELSE we need to send the data valid signal to request new block to be read */
		value = (char**)malloc(sizeof(char*)*1);
		value[0] = (char*)malloc(sizeof(char)*2);
		sprintf(value[0], "1");

		add_event_to_event_list(this_workload, current_time, INPUT_DATA_VALID, NULL, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

		num_blocks --;
	}
	else
	{
		value = (char**)malloc(sizeof(char*)*1);
		value[0] = (char*)malloc(sizeof(char)*2);
		sprintf(value[0], "1");

		resource_name = (char*)malloc(sizeof(char)*18);
		sprintf(resource_name, "input_termination");

		add_event_to_event_list(this_workload, current_time, -1, resource_name, value, 1, INPUT_EVENT, environment.minimum_arrival_rate_on_serial_interfaces);

		num_blocks --;

		is_new_block = TRUE;
	}
}
